#!/usr/bin/env python
#Copyright Bob Blair, 2010-2012
#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, distribute with modifications, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#Except as contained in this notice, the name(s) of the above copyright holders shall not be used in advertising or otherwise to promote the sale, use or other dealings in this Software without prior written authorization.
#

# Generate the main indexes, which have entries within subject.
# This version creates one file per subject.  Although there are thousands
# of subjects, this approach (on balance) creates a better user experience.
# Instead of creating a 2MB index report (which is generated by indexreport.py),
# this script generates much smaller files that are faster to retrieve.  The
# performace improvement offsets the loss of searchability through the
# subjects and entries.

import sys
import re
from indexutil import slist, getURL, roman2int

def set_headstrs(subj):
  strs = []
  strs.append('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"')
  strs.append('        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')
  strs.append('<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">')
  strs.append('<head><title>')
  strs.append(subj + ' in Gardiner\'s History of England')
  strs.append('</title>')
  strs.append('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>')
  strs.append('<link href="index.css" rel="stylesheet" type="text/css"/>')
  strs.append('</head><body>')
  strs.append('<h1 align=\"center\">')
  strs.append('Index of Gardiner\'s History of England')
  strs.append('</h1>')
  strs.append('<h2 align=\"center\">')
  strs.append(subj)
  strs.append('</h2><ul>')
  return strs

def set_tailstrs():
  strs = []
  strs.append('</ul></body></html>')
  return strs

def tell_usage():
  sys.stderr.write('This script generates index files for subjects in Gardiner\'s\n')
  sys.stderr.write('History of England by reading and processing a text file\n')
  sys.stderr.write('that contains the index data.  You must specify the name\n')
  sys.stderr.write('of the text file as the first command line argument.\n')
  sys.stderr.write('A second, optional, parameter is the path of a directory\n')
  sys.stderr.write('where the subject indexes will be written.\n\n')
  sys.stderr.write('About 2500 index files will be written.\n')
  return

def openSubjectReport(outdir, idx, subj):
  filename = outdir + '/S' + str(idx) + '.' + 'html'
  f = open(filename, 'w')
  if not f:
    print 'Failed to open file', filename, 'for output'
  print >>f, ''.join(set_headstrs(subj))
  return f

def closeSubjectReport(f):
  print >>f, ''.join(set_tailstrs())
  f.close()

def generate_html(index, outdir):
  #initialize
  subject = ""
  subjectcnt = 0
  lastvol = 0 # lastvol is used to process entries that don't have a volume
  lastpage = 0 # lastpage is used to process ibid. entries
  # create a list of compiled regular expressions for parsing out pieces
  # of the input file
  plist = [];
  for s in slist:
    plist.append(re.compile(s))
  # parse each line in the input
  for line in index:
    # skip blank lines (slist[5] matches a blank line)
    m = re.match(plist[5], line) # check blank line
    if (m):
      continue
    # Process a new subject
    m = re.match(plist[0], line) # check start of subject
    if (m):
      if subjectcnt > 0:
        closeSubjectReport(f)
      f = openSubjectReport(outdir, subjectcnt, line)
      subjectcnt = subjectcnt + 1
      lastvol = 0
      lastpage = 0
      continue
    # Process a reference
    m = re.match(plist[1], line) # check "See xxx"
    if m:
      print >>f, '<li>', m.group(0), '</li>'
      continue
    # Process an "ibid."
    m = re.match(plist[2], line) # check "xxx ib."
    if m:
      s = '<li><a href=\"' + getURL(lastvol, lastpage) + '\">'
      s += m.group(1) +  '</a></li>'
      print >>f, s
      continue
    # process a normal index entry under a subject, with a volume specified
    m = re.match(plist[3], line) # check "yada yada iii. 99"
    if m:
      vol = roman2int(m.group(2))
      page = int(m.group(3))
      s = '<li><a href=\"' + getURL(vol, page) + '\">'
      s += m.group(1) +  '</a></li>'
      print >>f, s
      lastvol = vol
      lastpage = page
      continue
 # process a normal index entry under a subject, without a volume specified
    m = re.match(plist[4], line) # check xxxx 99
    if (m):
      page = int(m.group(2))
      s = '<li><a href=\"' + getURL(lastvol, page) + '\">'
      s += m.group(1) +  '</a></li>'
      print >>f, s
      lastpage = page
      continue
  closeSubjectReport(f)
  return

def main():
  # open the input file and suck the whole thing in and print the html document
  if len(sys.argv) > 1 and sys.argv[1] == '?':
    tell_usage()
  else:
    f = open(sys.argv[1])
    index = f.readlines();
    f.close()
    if len(sys.argv) > 2:
      outdir = sys.argv[2]
    else:
      outdir = '.'
    generate_html(index, outdir)


if __name__ == '__main__':
  main()
