#!/usr/bin/env python
#Copyright Bob Blair, 2010-2012
#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, distribute with modifications, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#Except as contained in this notice, the name(s) of the above copyright holders shall not be used in advertising or otherwise to promote the sale, use or other dealings in this Software without prior written authorization.
#

# Print a report from the index showing the detail entry by volume by page.

# Change log.
#   1/21/2012  Add document type and css link
#   2/04/2012  Fixed tailstrs to close body instead of open a new one

import sys
import re
from indexutil import slist, getURL, roman2int

def set_headstrs():
  strs = []
  strs.append('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"')
  strs.append('        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">')
  strs.append('<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">')
  strs.append('<head><title>')
  strs.append('Index by volume and page of Samuel R. Gardiner\'s History of England')
  strs.append('</title>')
  strs.append('<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>')
  strs.append('<link href="index.css" rel="stylesheet" type="text/css"/>')
  strs.append('</head><body>')
  strs.append('<h1 align="center">')
  strs.append('Index by Volume and Page of Gardiner\'s History of England')
  strs.append('</h1>')
  strs.append('<table border="1" cellspacing="0" cellpadding="6"><tbody>')
  return strs

def set_tailstrs():
  return ['</tbody></table></body></html>']

# Add a structure to the list
def addlisting(list, subject, entry, vol, page, cnt):
  listing = [subject, entry, int(vol), int(page), int(cnt)]
  # print 'Adding', listing
  list.append(listing)

# Output a volume header.  The volume number is in vol.
def printvolume(vol):
  print '<tr>'
  print '<th colspan=\"3\" align=\"center\">Volume', str(vol), '</th>'
  print '</tr>'

# Output all the entries for a single page.  The entries are in pagelistings,
# which is a list of the structure type described above.
def printpage(pagelistings):
  print '<tr>'
  print '<td align=\"right\" valign=\"center\"> <a href=\"' + getURL(pagelistings[0][2], pagelistings[0][3]) + '\">' + str(pagelistings[0][3]) + '</a></td>' 
  print '<td>'
  pcnt = 0
  for plisting in pagelistings:
    if pcnt > 0:
      print '<br/>'
    pcnt = 1
    print '<a href=\"subjects/S' + str(plisting[4]) + '.html\">' + plisting[0].strip().rstrip('.,') + '</a>, ' + plisting[1]
  print '</td></tr>'
 
# print the contents of a volume, by page.  All the listings for the volume
# are in vollistings.
def printvollist(vollistings):
  vollistings.sort(key=lambda list: list[3]) # sort by page number
  lastpage = vollistings[0][3]
  pagelistings = []
  for listing in vollistings:
    #print listing
    if (listing[3] != lastpage):
      printpage(pagelistings)
      pagelistings = []
      lastpage = listing[3]
    pagelistings.append(listing)
  printpage(pagelistings)

# The strategy of this script is to accumulate a list of structures that
# contain the index entry and can be sorted by volume and page.  The
# structure is a tuple of
#    subject -- The index subject, i.e. "Cromwell, Oliver"
#    entry -- the index item for the subject, i.e. "birth"
#    vol -- the volume in which the entry occurs
#    page -- the page of the volume in which the entry occurs
#    cnt -- the id of the subject index

def accumulate_listings(index):
  # initialize
  plist = []
  listings = []
  subject = ""
  subjectcnt = 0
  lastvol = 0
  lastpage = 0
  # create a list of compiled regular expressions for parsing out pieces
  # of the input file
  for s in slist:
    plist.append(re.compile(s))
  for line in index:
    # skip blank lines
    m = re.match(plist[5], line) # check blank line
    if (m):
      continue
    # handle start of a new subject
    m = re.match(plist[0], line) # check start of subject
    if (m):
      subject = line.strip().rstrip('.,')
      subjectcnt = subjectcnt + 1
      lastvol = 0
      lastpage = 0
      continue
    # skip references
    m = re.match(plist[1], line) # check "See xxx"
    if m:
      continue
    m = re.match(plist[2], line) # check "xxx ib."
    if m:
      listings.append([subject, m.group(1), lastvol, lastpage, subjectcnt-1])
      continue
    m = re.match(plist[3], line) # check "xxx iii. 99"
    if m:
      vol = roman2int(m.group(2))
      page = int(m.group(3))
      listings.append([subject, m.group(1), vol, page, subjectcnt-1])
      lastvol = vol
      lastpage = page
      continue
    m = re.match(plist[4], line) # check xxxx 99
    if (m):
      page = int(m.group(2))
      listings.append([subject, m.group(1), lastvol, page, subjectcnt-1])
      lastpage = page
      continue
  return listings

def generate_html(index):
  # Accumulate all the index entries in memory, sorted by page within volume
  # The sort key is just volume, because in the text file from which the
  # listings are created, the listings are already in order by page
  # within volume.
  listings = accumulate_listings(index)
  listings.sort(key=lambda listing: listing[2])
  # print the html head
  print ''.join(set_headstrs())
  # print the first volume header
  vol = (listings[0])[2]
  printvolume(vol)
  # accumulate listings by volume (they are in order by volume by page)
  vollistings = []
  for listing in listings:
    # When the next volume is encountered, print the previous vol's listings
    # and start a new volume
    if listing[2] != vol:
      printvollist(vollistings)
      vol = listing[2]
      vollistings = []
      printvolume(vol)
    vollistings.append(listing)
  # print the listings for the last volume
  printvollist(vollistings)
  # print the html tail
  print ''.join(set_tailstrs())

def tell_usage():
  sys.stderr.write('This script generates a page-by-page subject index of Gardiner\'s\n')
  sys.stderr.write('History of England by reading and processing a text file\n')
  sys.stderr.write('that contains the index data.  You must specify the name\n')
  sys.stderr.write('of the text file as the first command line argument.\n')
  return

def main():
  # open the input file and suck the whole thing in and print the html document
  if len(sys.argv) > 1 and sys.argv[1] != '?':
    f = open(sys.argv[1])
    index = f.readlines();
    f.close()
    generate_html(index)
  else:
    tell_usage()

if __name__ == '__main__':
  main()
